#!/usr/bin/env python
# coding: utf-8

from cchardet import _cchardet

def detect(msg):
    """
    Args:
        msg: str
    Returns:
        {
            "encoding": str,
            "confidence": float
        }
    """
    encoding, confidence = _cchardet.detect_with_confidence(msg)
    if isinstance(encoding, bytes):
        encoding = encoding.decode()
    return {"encoding": encoding, "confidence": confidence}



#!/usr/bin/env python
# coding: utf-8

# nosetests --nocapture tests.py
from nose.tools import eq_

# http://docs.python.org/library/codecs.html
# https://bitbucket.org/medoc/uchardet-enhanced/src/85fc77c3e058/libcharsetdetect/README.md

# Support codecs
# Big5
# EUC-JP
# EUC-KR
# GB18030
# gb18030
# HZ-GB-2312
# IBM855
# IBM866
# ISO-2022-CN
# ISO-2022-JP
# ISO-2022-KR
# ISO-8859-2
# ISO-8859-5
# ISO-8859-7
# ISO-8859-8
# KOI8-R
# Shift_JIS
# TIS-620
# UTF-8
# UTF-16BE
# UTF-16LE
# UTF-32BE
# UTF-32LE
# windows-1250
# windows-1251
# windows-1252
# windows-1253
# windows-1255
# x-euc-tw
# X-ISO-10646-UCS-4-2143
# X-ISO-10646-UCS-4-3412
# x-mac-cyrillic
import cchardet

class TestCchardet():

    def test_detect_bg_iso88595(self):
        encoding = "ISO-8859-5"
        path = r"testdata/bg/ISO-8859-5/wikitop_bg_ISO-8859-5.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_bg_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/bg/UTF-8/wikitop_bg_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_bg_windows1251(self):
        encoding = "WINDOWS-1251"
        path = r"testdata/bg/WINDOWS-1251/wikitop_bg_WINDOWS-1251.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_cz_iso88592(self):
        encoding = "ISO-8859-2"
        path = r"testdata/cz/ISO-8859-2/wikitop_cz_ISO-8859-2.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_cz_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/cz/UTF-8/wikitop_cz_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_de_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/de/UTF-8/wikitop_de_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_de_windows1252(self):
        encoding = "WINDOWS-1252"
        path = r"testdata/de/WINDOWS-1252/wikitop_de_WINDOWS-1252.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_dk_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/dk/UTF-8/wikitop_dk_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_dk_windows1252(self):
        encoding = "WINDOWS-1252"
        path = r"testdata/dk/WINDOWS-1252/wikitop_dk_WINDOWS-1252.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_el_iso88597(self):
        encoding = "ISO-8859-7"
        path = r"testdata/el/ISO-8859-7/wikitop_el_ISO-8859-7.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_el_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/el/UTF-8/wikitop_el_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_en_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/en/UTF-8/wikitop_en_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_en_windows1252(self):
        encoding = "WINDOWS-1252"
        path = r"testdata/en/WINDOWS-1252/wikitop_en_WINDOWS-1252.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_es_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/es/UTF-8/wikitop_es_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_es_windows1252(self):
        encoding = "WINDOWS-1252"
        path = r"testdata/es/WINDOWS-1252/wikitop_es_WINDOWS-1252.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_fi_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/fi/UTF-8/wikitop_fi_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_fi_windows1252(self):
        encoding = "WINDOWS-1252"
        path = r"testdata/fi/WINDOWS-1252/wikitop_fi_WINDOWS-1252.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_fr_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/fr/UTF-8/wikitop_fr_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_fr_windows1252(self):
        encoding = "WINDOWS-1252"
        path = r"testdata/fr/WINDOWS-1252/wikitop_fr_WINDOWS-1252.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_he_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/he/UTF-8/wikitop_he_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_he_windows1255(self):
        encoding = "WINDOWS-1255"
        path = r"testdata/he/WINDOWS-1255/wikitop_he_WINDOWS-1255.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_hu_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/hu/UTF-8/wikitop_hu_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_hu_iso55892(self):
        encoding = "ISO-8859-2"
        path = r"testdata/hu/ISO-8859-2/wikitop_hu_ISO-8859-2.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_it_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/it/UTF-8/wikitop_it_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_it_windows1252(self):
        encoding = "WINDOWS-1252"
        path = r"testdata/it/WINDOWS-1252/wikitop_it_WINDOWS-1252.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_nl_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/nl/UTF-8/wikitop_nl_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_nl_windows1252(self):
        encoding = "WINDOWS-1252"
        path = r"testdata/nl/WINDOWS-1252/wikitop_nl_WINDOWS-1252.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_no_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/no/UTF-8/wikitop_no_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_no_windows1252(self):
        encoding = "WINDOWS-1252"
        path = r"testdata/no/WINDOWS-1252/wikitop_no_WINDOWS-1252.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_pl_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/pl/UTF-8/wikitop_pl_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_pl_iso88592(self):
        encoding = "ISO-8859-2"
        path = r"testdata/pl/ISO-8859-2/wikitop_pl_ISO-8859-2.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_pt_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/pt/UTF-8/wikitop_pt_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_pt_windows1252(self):
        encoding = "WINDOWS-1252"
        path = r"testdata/pt/WINDOWS-1252/wikitop_pt_WINDOWS-1252.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_ru_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/ru/UTF-8/wikitop_ru_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_ru_windows1251(self):
        encoding = "WINDOWS-1251"
        path = r"testdata/ru/WINDOWS-1251/wikitop_ru_WINDOWS-1251.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_ru_ibm855(self):
        encoding = "IBM855"
        path = r"testdata/ru/IBM855/wikitop_ru_IBM855.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_ru_koi8r(self):
        encoding = "KOI8-R"
        path = r"testdata/ru/KOI8-R/wikitop_ru_KOI8-R.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_ru_maccyrillic(self):
        encoding = "MAC-CYRILLIC"
        path = r"testdata/ru/X-MAC-CYRILLIC/wikitop_ru_MACCYRILLIC.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_se_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/se/UTF-8/wikitop_se_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_se_windows1252(self):
        encoding = "WINDOWS-1252"
        path = r"testdata/se/WINDOWS-1252/wikitop_se_WINDOWS-1252.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_th_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/th/UTF-8/wikitop_th_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_th_tis620_1(self):
        encoding = "TIS-620"
        path = r"testdata/th/TIS-620/utffool_th_TIS-620.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_th_tis620_2(self):
        encoding = "TIS-620"
        path = r"testdata/th/TIS-620/wikitop_th_TIS-620.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_tr_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/tr/UTF-8/wikitop_tr_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_tr_iso88599(self):
        encoding = "ISO-8859-9"
        path = r"testdata/tr/ISO-8859-9/wikitop_tr_ISO-8859-9.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_zh_utf8(self):
        encoding = "UTF-8"
        path = r"testdata/zh/UTF-8/wikitop_zh_UTF-8.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        print(detected_encoding)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

    def test_detect_zh_gb18030(self):
        encoding = "GB18030"
        path = r"testdata/zh/GB18030/wikitop_zh_GB18030.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        detected_encoding = cchardet.detect(msg)
        eq_(encoding.lower(),detected_encoding['encoding'].lower())

class TestCchardetSpeed():
    def test_speed(self):
        try:
            import chardet
            has_chardet = True
        except ImportError:
            has_chardet = False
        import time
        do_times = 5
        path = r"testdata/wikipediaJa_One_Thousand_and_One_Nights_SJIS.txt"
        with open(path, 'rb') as f:
            msg = f.read()
        # Test chardet
        if has_chardet:
            result_chardet = 0
            for i in range(do_times):
                start_chardet = time.time()
                chardet.detect(msg)
                result_chardet += (time.time() - start_chardet)
            print('chardet:',1/(result_chardet/do_times), 'call(s)/s')
        # Test cchardet
        result_cchardet = 0
        for i in range(do_times):
            start_cchardet = time.time()
            cchardet.detect(msg)
            result_cchardet += (time.time() - start_cchardet)
        print('cchardet:',1/(result_cchardet/do_times), 'call(s)/s')



from libc.stdlib cimport malloc, free
import warnings

cdef extern from "Python.h":
    void * PyMem_Malloc(size_t)
    void PyMem_Free(void *)

cdef extern from *:
    ctypedef char* const_char_ptr "const char*"

cdef extern from "string.h":
    cdef int strlen(char *s)

cdef extern from "charsetdetect.h":
    ctypedef void* csd_t
    cdef csd_t csd_open()
    cdef int csd_consider(csd_t csd, char* data, int length)
    cdef const_char_ptr csd_close(csd_t csd)
    cdef const_char_ptr csd_close2(csd_t csd, float *confidence)

def detect(char *msg):
    cdef csd_t csd = csd_open()
    cdef int length = strlen(msg)
    cdef int result = csd_consider(csd, msg, length)
    # ref: charsetdetect.cpp
    if result == -1: # Error, signal with a negative number
        raise Exception("Error, signal with a negative number")
    elif result == 1: # Need more data
        warnings.warn("Need more data",UserWarning)
        ret = csd_close(csd)
    elif result == 0: # Detected early
        ret = csd_close(csd)
    if ret:
        return ret

def detect_with_confidence(char *msg):
    cdef csd_t csd = csd_open()
    cdef int length = strlen(msg)
    cdef int result = csd_consider(csd, msg, length)
    cdef float confidence = 0.0
    cdef const_char_ptr detected_charset
    # ref: charsetdetect.cpp
    if result == 1: # Need more data
        detected_charset = csd_close2(csd, &confidence)
    elif result == 0: # Detected early
        detected_charset = csd_close2(csd, &confidence)
    else: # Error, signal with a negative number
        raise Exception("Error, signal with a negative number")
    if detected_charset:
        return detected_charset, confidence
    else:
        return None, None



#!/usr/bin/env python
# coding: utf-8

# python setup.py sdist --formats=gztar

import os
import sys
import platform

try:
    from setuptools import setup, Extension
except ImportError:
    from distutils.core import setup, Extension

import glob
try:
    import Cython.Compiler.Main as cython_compiler
    have_cython = True
except ImportError:
    have_cython = False
from distutils.command.build_ext import build_ext

try:
    import pypissh
except ImportError:
    # non-developer
    pass
else:
    pypissh.monkeypatch()

DEBUG = False

src_dir = 'src'
ext_dir = os.path.join(src_dir,'ext')
build_dir = 'build'
cchardet_dir = os.path.join(src_dir,'cchardet/')
charsetdetect_dir = os.path.join(ext_dir, 'libcharsetdetect/')
nspr_emu_dir = os.path.join(charsetdetect_dir,"nspr-emu/")
uchardet_dir = os.path.join(charsetdetect_dir,"mozilla/extensions/universalchardet/src/base/")

if have_cython:
    pyx_sources = glob.glob(cchardet_dir+'*.pyx')
    sys.stderr.write("cythonize: %r\n" % (pyx_sources,))
    cython_compiler.compile(pyx_sources,options=cython_compiler.CompilationOptions(cplus=True))
cchardet_sources = glob.glob(cchardet_dir+'*.cpp')
sources = cchardet_sources  + [os.path.join(charsetdetect_dir,"charsetdetect.cpp")] + glob.glob(uchardet_dir+'*.cpp')

macros = []
extra_compile_args = []
extra_link_args = []

if platform.system() == "Windows":
    macros.append(("WIN32","1"))

if DEBUG:
    macros.append(("DEBUG_chardet","1"))
    extra_compile_args.append("-g"),
    extra_link_args.append("-g"),

cchardet_module = Extension("cchardet._cchardet",
    sources = sources,
    include_dirs = [uchardet_dir,nspr_emu_dir,charsetdetect_dir],
    language = "c++",
    define_macros=macros,
)

setup(
    name = 'cchardet',
    author = 'PyYoshi',
    author_email = 'myoshi321go_at_gmail_dot_com',
    url = r"https://github.com/PyYoshi/cChardet",
    description = 'Universal encoding detector. This library is faster than chardet.',
    long_description= """cChardet is high speed universal character encoding detector. - binding to charsetdetect.
This library is faster than chardet.
""",
    version = '1.0.0',
    license = 'MIT License',
    classifiers = [
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Cython',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
    ],
    keywords = [
        'cython',
        'chardet',
        'charsetdetect'
    ],
    cmdclass = {'build_ext': build_ext},
    package_dir = {"":src_dir},
    packages = ['cchardet',],
    ext_modules = [
        cchardet_module
    ],
)
